#ifndef _DRV_TAPI_OSMAP_H
#define _DRV_TAPI_OSMAP_H
/****************************************************************************

                              Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 85579 Neubiberg, Germany

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

 ****************************************************************************
   Module      : drv_tapi_osmap.h
   Description : This file contains the includes and the defines
                 specific to the OS

*******************************************************************************/

#if defined(LINUX)
   #include <linux/kernel.h>
   #ifdef MODULE
      #include <linux/module.h>
   #endif
   #include <linux/version.h>
   #if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
      #include <linux/hardirq.h>
   #else
      #include <asm/hardirq.h>
   #endif /* Ver >= 2.6 */
   #include <asm/byteorder.h>
   #include <linux/interrupt.h>
#endif /* LINUX */

#include "ifx_types.h"     /* ifx type definitions */

#ifndef HAVE_IFX_ULONG_T
   #warning please update your ifx_types.h, using local definition of IFX_ulong_t
   /* unsigned long type - valid for 32bit systems only */
   typedef unsigned long               IFX_ulong_t;
   #define HAVE_IFX_ULONG_T
#endif /* HAVE_IFX_ULONG_T */

#ifndef HAVE_IFX_LONG_T
   #warning please update your ifx_types.h, using local definition of IFX_long_t
   /* long type - valid for 32bit systems only */
   typedef long                        IFX_long_t;
   #define HAVE_IFX_LONG_T
#endif /* HAVE_IFX_LONG_T */

#ifndef HAVE_IFX_INTPTR_T
   #warning please update your ifx_types.h, using local definition of IFX_intptr_t
   typedef IFX_long_t                  IFX_intptr_t;
   #define HAVE_IFX_INTPTR_T
#endif /* HAVE_IFX_INTPTR_T */

#ifndef HAVE_IFX_SIZE_T
   #warning please update your ifx_types.h, using local definition of IFX_size_t
   typedef IFX_ulong_t                 IFX_size_t;
   #define HAVE_IFX_SIZE_T
#endif /* HAVE_IFX_SIZE_T */

#ifndef HAVE_IFX_SSIZE_T
   #warning please update your ifx_types.h, using local definition of IFX_ssize_t
   typedef IFX_long_t                  IFX_ssize_t;
   #define HAVE_IFX_SSIZE_T
#endif /* HAVE_IFX_SSIZE_T */

#include "ifxos_interrupt.h"
#include "ifxos_thread.h"

/* ============================= */
/* Global Defines                */
/* ============================= */
#ifndef IFXOS_THREAD_PRIORITY_MODIFY
   #ifdef LINUX
      #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
         #define __linux_setscheduler(param) \
            setscheduler (current->pid, SCHED_FIFO, param)
      #else
         #define __linux_setscheduler(param) \
            sched_setscheduler (current, SCHED_FIFO, param)
      #endif /* LINUX_2_4 */

      #define IFXOS_THREAD_PRIORITY_MODIFY(prio) \
         do { \
            struct sched_param sched_params; \
            IFX_int32_t ret; \
            sched_params.sched_priority = (prio); \
            ret = __linux_setscheduler (&sched_params); \
            if (ret < 0) \
            { \
               printk("kthread error, real time priority %d couln't be set," \
                      " ret=%d\n\r", prio, ret); \
            } \
         } while(0)

   #endif /* LINUX */
   #ifdef VXWORKS
      #define IFXOS_THREAD_PRIORITY_MODIFY(prio) \
            taskPrioritySet(taskIdSelf(), prio)
   #endif /* VXWORKS */
#endif /* IFXOS_THREAD_PRIORITY_MODIFY */

#ifdef LINUX
   #define IFXOS_THREAD_KILL(pThrCntrl) \
      do { \
         if((pThrCntrl) && \
            (IFXOS_THREAD_INIT_VALID(pThrCntrl) == IFX_TRUE) && \
            ((pThrCntrl)->thrParams.bRunning == 1)) { \
               /* trigger user thread routine to shutdown */ \
               (pThrCntrl)->thrParams.bShutDown = IFX_TRUE; \
               /* this function needs to be protected with the big \
                  kernel lock (lock_kernel()). The lock must be \
                  grabbed before changing the terminate \
                  flag and released after the down() call. */ \
               lock_kernel(); \
               mb(); \
               kill_proc((pThrCntrl)->tid, SIGKILL, 1); \
               /* release the big kernel lock */ \
               unlock_kernel(); \
               wait_for_completion (&(pThrCntrl)->thrCompletion); \
               /* now we are sure the thread is in zombie state. We \
                  notify keventd to clean the process up. \
               */ \
               kill_proc(2, SIGCHLD, 1); \
         } \
      } while (0)
#else
   #define IFXOS_THREAD_KILL(pThrCntrl)
#endif /* LINUX */

#endif /* _DRV_TAPI_OSMAP_H */

